Index: m_map.c
===================================================================
RCS file: /cvsroot/ircd-ratbox/modules/m_map.c,v
retrieving revision 1.36
diff -u -r1.36 m_map.c
--- m_map.c	3 Mar 2004 23:03:04 -0000	1.36
+++ m_map.c	9 Aug 2004 16:00:10 -0000
@@ -44,6 +44,7 @@
 DECLARE_MODULE_AV1(map, NULL, NULL, map_clist, NULL, NULL, "$Revision: 1.36 $");
 
 static void dump_map(struct Client *client_p, struct Client *root, char *pbuf);
+static void flattened_map(struct Client *client_p);
 
 static char buf[BUFSIZE];
 
@@ -53,10 +54,10 @@
 static int
 m_map(struct Client *client_p, struct Client *source_p, int parc, const char *parv[])
 {
-	if((!IsExemptShide(source_p) && ConfigServerHide.flatten_links) ||
-	   ConfigFileEntry.map_oper_only)
+	if(!IsExemptShide(source_p) && ConfigServerHide.flatten_links)
 	{
-		m_not_oper(client_p, source_p, parc, parv);
+		flattened_map(client_p);
+		sendto_one(client_p, form_str(RPL_MAPEND), me.name, client_p->name);
 		return 0;
 	}
 
@@ -145,3 +146,72 @@
 		i++;
 	}
 }
+
+/*
+** flattened_map - display a version of map that doesn't give away routing
+**                 information to users when flattened links is enabled.
+*/
+static void
+flattened_map(struct Client *client_p)
+{
+	char buf[BUFSIZE];
+	dlink_node *ptr;
+	struct Client *target_p;
+	int i, len;
+	int cnt = 0;
+
+	/* First display me as the root */
+	strlcpy(buf, me.name, BUFSIZE);
+	len = strlen(buf);
+	buf[len] = ' ';
+
+	if(len < USER_COL)
+	{
+		for (i = len + 1; i < USER_COL; i++)
+		{
+			buf[i] = '-';
+		}
+	}
+
+	ircsnprintf(buf + USER_COL, BUFSIZE - USER_COL,
+		" | Users: %5lu (%4.1f%%)", dlink_list_length(&me.serv->users),
+		100 * (float) dlink_list_length(&me.serv->users) / (float) Count.total);
+
+	sendto_one(client_p, form_str(RPL_MAP), me.name, client_p->name, buf);
+
+	/* Next, we run through every other server and list them */
+	DLINK_FOREACH(ptr, global_serv_list.head)
+	{
+		target_p = ptr->data;
+
+		cnt++;
+
+		/* Skip ourselves, it's already counted */
+		if(IsMe(target_p) ||
+		    (IsHidden(target_p) && !ConfigServerHide.disable_hidden))
+			continue;
+
+		if (cnt == dlink_list_length(&global_serv_list))
+			strlcpy(buf, "`- ", BUFSIZE);
+		else
+			strlcpy(buf, "|- ", BUFSIZE);
+
+		strlcat(buf, target_p->name, BUFSIZE);
+		len = strlen(buf);
+		buf[len] = ' ';
+
+		if(len < USER_COL)
+		{
+			for (i = len + 1; i < USER_COL; i++)
+			{
+				buf[i] = '-';
+			}
+		}
+
+		ircsnprintf(buf + USER_COL, BUFSIZE - USER_COL,
+			" | Users: %5lu (%4.1f%%)", dlink_list_length(&target_p->serv->users),
+			100 * (float) dlink_list_length(&target_p->serv->users) / (float) Count.total);
+
+		sendto_one(client_p, form_str(RPL_MAP), me.name, client_p->name, buf);
+	}
+}
